#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <termio.h>
#include <string.h>

#define TTY "/dev/ttyS0"

#define msleep(p) usleep((p) * 1000)

static int g_fd;

void RightSwitchOn(void)
{
	printf("RightSwitchOn\n");
}

void RightSwitchOff(void)
{
	printf("RightSwitchOff\n");
}

void LeftSwitchOn(void)
{
	printf("LeftSwitchOn\n");
}
void LeftSwitchOff(void)
{
	printf("LeftSwitchOff\n");
}

unsigned char serial_data(void);
void enocean_loop(void);

int main(int ac, char **av)
{
	int fd;
	int len;
	struct termios tio;

	printf("EnOcean serial receiver started.\n");

	if ((fd = open(TTY, O_RDWR)) < 0) {
		fprintf(stderr, "%s: open error\n", TTY);
		exit(1);
	}
	bzero(&tio, sizeof(tio));
	tio.c_cflag = B9600 | CS8 | CLOCAL | CREAD;
	tio.c_cc[VMIN] = 1;
	tcsetattr(fd, TCSANOW, &tio);
	g_fd = fd;

	while(1) {
	      enocean_loop();
	}
	return 0;
}

unsigned char serial_data(void)
{
	unsigned char buffer[2];
	int len;
	
	len = read(g_fd, buffer, 1);
	if (len <= 0) {
		fprintf(stderr, "read error = %d\n", len);
		return len;
	}
	return(buffer[0]);
}

void enocean_loop(void)
{
	char buffer[32];
	char data[2];
	unsigned char id[4];
	int len;
	int seq;
	unsigned short sum;
	int org;
	int gotHeader = 0;
	int nu;
	unsigned char checkSum;
	int i;

	while(!gotHeader) {
	        if (serial_data() == 0xA5) {
			gotHeader = serial_data() == 0x5A;
		}
		else {
			msleep(10);
		}
	}

	sum = serial_data();
	seq = sum >> 5;
	len = sum & 0x1F;
	for(i = 0; i < len - 1; i++) {
		buffer[i] = serial_data();
		sum += buffer[i];
	}
	checkSum = serial_data();
  
	org = buffer[0];
	id[0] = buffer[5];
	id[1] = buffer[6];
	id[2] = buffer[7];
	id[3] = buffer[8];
	nu = (buffer[9] >> 4) & 0x01;
	data[0] = buffer[1] >> 4;
	data[1] = buffer[1] & 0x0F;

	printf("org=%02x, id=%02X%02X%02X%02X, dat=%02X sum=%02X, chk=%02X\n",
	       org,
	       id[0], id[1], id[2], id[3], buffer[1], 
	       sum, checkSum);
  
	if (checkSum == (sum & 0xFF)) {
		//if (org == 5 && id == 0x001A4351 && nu == 0x01) {
		if (org == 5 && nu == 0x01) {
			for (i = 0; i < 2; i++) {
				switch (data[i]) {
				case 0x01:
					RightSwitchOn();
					break;
				case 0x03:
					RightSwitchOff();
					break;
				case 0x05:
					LeftSwitchOn();
					break;
				case 0x07:
					LeftSwitchOff();
					break;
				} // switch
			} //for
		} //if(org == 5 ...
	} // if (checkSum
	else {
		fprintf(stderr, "sum-error.\n");
	}
}

